import super_gradients
from super_gradients import init_trainer, Trainer
from super_gradients.training.utils.distributed_training_utils import setup_device
from super_gradients.training import models, dataloaders
from super_gradients.common.data_types.enum import MultiGPUMode, StrictLoad
import super_gradients


def main():
    init_trainer()
    setup_device(device=None, multi_gpu="DDP", num_gpus=1)

    trainer = Trainer(experiment_name="estop_set", ckpt_root_dir="None")

    num_classes = 5
    arch_params = {
        "in_channels": 3,
        "backbone": {
            "NStageBackbone": {
                "stem": {"YoloNASStem": {"out_channels": 32}},
                "stages": [
                    {"YoloNASStage": {"out_channels": 64, "num_blocks": 2, "activation_type": "relu", "hidden_channels": 32, "concat_intermediates": False}},
                    {"YoloNASStage": {"out_channels": 128, "num_blocks": 3, "activation_type": "relu", "hidden_channels": 48, "concat_intermediates": False}},
                    {"YoloNASStage": {"out_channels": 256, "num_blocks": 4, "activation_type": "relu", "hidden_channels": 64, "concat_intermediates": False}},
                    {"YoloNASStage": {"out_channels": 512, "num_blocks": 2, "activation_type": "relu", "hidden_channels": 128, "concat_intermediates": False}},
                ],
                "context_module": {"SPP": {"output_channels": 512, "activation_type": "relu", "k": [5, 9, 13]}},
                "out_layers": ["stage1", "stage2", "stage3", "context_module"],
            }
        },
        "neck": {
            "YoloNASPANNeckWithC2": {
                "neck1": {
                    "YoloNASUpStage": {
                        "out_channels": 128,
                        "num_blocks": 2,
                        "hidden_channels": 48,
                        "width_mult": 1,
                        "depth_mult": 1,
                        "activation_type": "relu",
                        "reduce_channels": True,
                    }
                },
                "neck2": {
                    "YoloNASUpStage": {
                        "out_channels": 64,
                        "num_blocks": 2,
                        "hidden_channels": 32,
                        "width_mult": 1,
                        "depth_mult": 1,
                        "activation_type": "relu",
                        "reduce_channels": True,
                    }
                },
                "neck3": {
                    "YoloNASDownStage": {
                        "out_channels": 128,
                        "num_blocks": 2,
                        "hidden_channels": 48,
                        "activation_type": "relu",
                        "width_mult": 1,
                        "depth_mult": 1,
                    }
                },
                "neck4": {
                    "YoloNASDownStage": {
                        "out_channels": 256,
                        "num_blocks": 2,
                        "hidden_channels": 48,
                        "activation_type": "relu",
                        "width_mult": 1,
                        "depth_mult": 1,
                    }
                },
            }
        },
        "heads": {
            "YoloNASPoseNDFLHeads": {
                "num_classes": 5,
                "reg_max": 16,
                "pose_offset_multiplier": 1.0,
                "compensate_grid_cell_offset": True,
                "inference_mode": False,
                "heads_list": [
                    {
                        "YoloNASPoseDFLHead": {
                            "bbox_inter_channels": 128,
                            "pose_inter_channels": 128,
                            "pose_regression_blocks": 2,
                            "shared_stem": False,
                            "width_mult": 0.33,
                            "pose_conf_in_class_head": True,
                            "pose_block_use_repvgg": False,
                            "first_conv_group_size": 0,
                            "num_classes": None,
                            "stride": 8,
                            "reg_max": 16,
                        }
                    },
                    {
                        "YoloNASPoseDFLHead": {
                            "bbox_inter_channels": 256,
                            "pose_inter_channels": 512,
                            "pose_regression_blocks": 2,
                            "shared_stem": False,
                            "width_mult": 0.33,
                            "pose_conf_in_class_head": True,
                            "pose_block_use_repvgg": False,
                            "first_conv_group_size": 0,
                            "num_classes": None,
                            "stride": 16,
                            "reg_max": 16,
                        }
                    },
                    {
                        "YoloNASPoseDFLHead": {
                            "bbox_inter_channels": 512,
                            "pose_inter_channels": 512,
                            "pose_regression_blocks": 3,
                            "shared_stem": False,
                            "width_mult": 0.33,
                            "pose_conf_in_class_head": True,
                            "pose_block_use_repvgg": False,
                            "first_conv_group_size": 0,
                            "num_classes": None,
                            "stride": 32,
                            "reg_max": 16,
                        }
                    },
                ],
            }
        },
        "bn_eps": 1e-06,
        "bn_momentum": 0.03,
        "inplace_act": True,
        "_convert_": "all",
        "num_classes": 5,
    }

    model = models.get(
        model_name="yolo_nas_pose_n",
        num_classes=num_classes,
        arch_params=arch_params,
        strict_load=StrictLoad.NO_KEY_MATCHING,
        pretrained_weights=None,
        checkpoint_path=None,
        load_backbone=False,
        checkpoint_num_classes=None,
    )

    train_dataloader = dataloaders.get(
        name=None,
        dataset_params={
            "data_dir": "/home/skpawar1305/Desktop/synthetic_data_v3/dataset/",
            "images_dir": ".",
            "json_file": "train.json",
            "edge_links": [
                [0, 1],
                [0, 2],
                [1, 2],
                [1, 3],
                [2, 4],
            ],
            "edge_colors": [
                [214, 39, 40],
                [148, 103, 189],
                [44, 160, 44],
                [140, 86, 75],
                [227, 119, 194],
            ],
            "keypoint_colors": [
                [148, 103, 189],
                [31, 119, 180],
                [148, 103, 189],
                [31, 119, 180],
                [148, 103, 189],
            ],
            "include_empty_samples": True,
            "crowd_annotations_action": "mask_as_normal",
            "transforms": [
                {"KeypointsRandomHorizontalFlip": {"flip_index": [1, 0, 3, 2, 4], "prob": 0.5}},
                {"KeypointsBrightnessContrast": {"brightness_range": [0.8, 1.2], "contrast_range": [0.8, 1.2], "prob": 0.5}},
                {"KeypointsHSV": {"hgain": 20, "sgain": 20, "vgain": 20, "prob": 0.5}},
                {
                    "KeypointsRandomAffineTransform": {
                        "max_rotation": 5,
                        "min_scale": 0.75,
                        "max_scale": 1.5,
                        "max_translate": 0.1,
                        "image_pad_value": 127,
                        "mask_pad_value": 1,
                        "prob": 0.75,
                        "interpolation_mode": [0, 1, 2, 3, 4],
                    }
                },
                {"KeypointsMosaic": {"prob": 0.5}},
                {"KeypointsLongestMaxSize": {"max_height": 640, "max_width": 640}},
                {
                    "KeypointsPadIfNeeded": {
                        "min_height": 640,
                        "min_width": 640,
                        "image_pad_value": [127, 127, 127],
                        "mask_pad_value": 1,
                        "padding_mode": "center",
                    }
                },
                {"KeypointsImageStandardize": {"max_value": 255}},
                {"KeypointsRemoveSmallObjects": {"min_instance_area": 1, "min_visible_keypoints": 1}},
            ],
        },
        dataloader_params={
            "dataset": "COCOPoseEstimationDataset",
            "shuffle": True,
            "batch_size": 8,
            "num_workers": 4,
            "drop_last": True,
            "pin_memory": False,
            "collate_fn": "YoloNASPoseCollateFN",
        },
    )

    val_dataloader = dataloaders.get(
        name=None,
        dataset_params={
            "data_dir": "/home/skpawar1305/Desktop/synthetic_data_v3/dataset/",
            "images_dir": ".",
            "json_file": "val.json",
            "edge_links": [
                [0, 1],
                [0, 2],
                [1, 2],
                [1, 3],
                [2, 4],
            ],
            "edge_colors": [
                [214, 39, 40],
                [148, 103, 189],
                [44, 160, 44],
                [140, 86, 75],
                [227, 119, 194],
            ],
            "keypoint_colors": [
                [148, 103, 189],
                [31, 119, 180],
                [148, 103, 189],
                [31, 119, 180],
                [148, 103, 189],
            ],
            "include_empty_samples": True,
            "crowd_annotations_action": "no_action",
            "transforms": [
                {"KeypointsLongestMaxSize": {"max_height": 640, "max_width": 640}},
                {"KeypointsPadIfNeeded": {"min_height": 640, "min_width": 640, "image_pad_value": 127, "mask_pad_value": 1, "padding_mode": "bottom_right"}},
                {"KeypointsImageStandardize": {"max_value": 255}},
            ],
        },
        dataloader_params={
            "dataset": "COCOPoseEstimationDataset",
            "batch_size": 8,
            "num_workers": 4,
            "drop_last": False,
            "pin_memory": False,
            "collate_fn": "YoloNASPoseCollateFN",
        },
    )

    _lr_updates = super_gradients.training.utils.utils.empty_list()

    _valid_metrics_list_0_poseestimationmetrics_post_prediction_callback = (
        super_gradients.training.models.pose_estimation_models.yolo_nas_pose.YoloNASPosePostPredictionCallback(
            pose_confidence_threshold=0.01, nms_iou_threshold=0.7, pre_nms_max_predictions=300, post_nms_max_predictions=30
        )
    )

    _phase_callbacks_0_earlystop_phase = super_gradients.training.utils.callbacks.base_callbacks.Phase(value="VALIDATION_EPOCH_END")

    training_hyperparams = {
        "resume": False,
        "run_id": None,
        "resume_path": None,
        "resume_from_remote_sg_logger": False,
        "ckpt_name": "ckpt_latest.pth",
        "lr_mode": "cosine",
        "lr_schedule_function": None,
        "lr_warmup_epochs": 10,
        "lr_warmup_steps": 128,
        "lr_cooldown_epochs": 0,
        "warmup_initial_lr": 1e-06,
        "step_lr_update_freq": None,
        "cosine_final_lr_ratio": 0.05,
        "warmup_mode": "LinearBatchLRWarmup",
        "lr_updates": _lr_updates,
        "pre_prediction_callback": None,
        "optimizer": "AdamW",
        "optimizer_params": {"weight_decay": 1e-06},
        "load_opt_params": True,
        "zero_weight_decay_on_bias_and_bn": True,
        "loss": "yolo_nas_pose_loss",
        "criterion_params": {
            "oks_sigmas": [0.2, 0.2, 0.2, 0.2, 0.2],
            "classification_loss_weight": 1.0,
            "classification_loss_type": "focal",
            "regression_iou_loss_type": "ciou",
            "iou_loss_weight": 2.5,
            "dfl_loss_weight": 0.01,
            "pose_cls_loss_weight": 1.0,
            "pose_reg_loss_weight": 34.0,
            "pose_classification_loss_type": "focal",
            "rescale_pose_loss_with_assigned_score": True,
            "assigner_multiply_by_pose_oks": True,
        },
        "ema": True,
        "ema_params": {"decay": 0.997, "decay_type": "threshold", "beta": 15},
        "train_metrics_list": [],
        "valid_metrics_list": [
            {
                "PoseEstimationMetrics": {
                    "num_joints": 5,
                    "oks_sigmas": [0.2, 0.2, 0.2, 0.2, 0.2],
                    "max_objects_per_image": 30,
                    "post_prediction_callback": _valid_metrics_list_0_poseestimationmetrics_post_prediction_callback,
                }
            }
        ],
        "metric_to_watch": "AP",
        "greater_metric_to_watch_is_better": True,
        "launch_tensorboard": False,
        "tensorboard_port": None,
        "tb_files_user_prompt": False,
        "save_tensorboard_to_s3": False,
        "precise_bn": False,
        "precise_bn_batch_size": None,
        "sync_bn": False,
        "silent_mode": False,
        "mixed_precision": True,
        "save_ckpt_epoch_list": [300, 500],
        "average_best_models": True,
        "dataset_statistics": False,
        "batch_accumulate": 1,
        "run_validation_freq": 1,
        "run_test_freq": 1,
        "save_model": True,
        "seed": 42,
        "phase_callbacks": [
            {"EarlyStop": {"phase": _phase_callbacks_0_earlystop_phase, "monitor": "AP", "mode": "max", "min_delta": 0.0001, "patience": 100, "verbose": True}}
        ],
        "log_installed_packages": True,
        "clip_grad_norm": None,
        "ckpt_best_name": "ckpt_best.pth",
        "max_train_batches": None,
        "max_valid_batches": None,
        "sg_logger": "base_sg_logger",
        "sg_logger_params": {
            "tb_files_user_prompt": False,
            "launch_tensorboard": False,
            "tensorboard_port": None,
            "save_checkpoints_remote": False,
            "save_tensorboard_remote": False,
            "save_logs_remote": False,
            "monitor_system": True,
        },
        "torch_compile": False,
        "torch_compile_loss": False,
        "torch_compile_options": {"mode": "reduce-overhead", "fullgraph": False, "dynamic": False, "backend": "inductor", "options": None, "disable": False},
        "finetune": False,
        "_convert_": "all",
        "initial_lr": 0.002,
        "max_epochs": 100,
    }

    # TRAIN
    result = trainer.train(
        model=model,
        train_loader=train_dataloader,
        valid_loader=val_dataloader,
        training_params=training_hyperparams,
    )

    print(result)


if __name__ == "__main__":
    main()
